@@ -13,10 +13,12 @@
 #    under the License.
 
 from rally.common import cfg
+from rally.common import logging
 from rally.task import service
 
 
 CONF = cfg.CONF
+LOG = logging.getLogger(__name__)
 
 
 Volume = service.make_resource_cls(
@@ -42,7 +44,7 @@ class BlockStorage(service.UnifiedService):
                       volume_type=None, user_id=None,
                       project_id=None, availability_zone=None,
                       metadata=None, imageRef=None, scheduler_hints=None,
-                      source_replica=None, multiattach=False):
+                      source_replica=None, multiattach=False, backup_id=None):
         """Creates a volume.
 
         :param size: Size of volume in GB
@@ -58,14 +60,18 @@ class BlockStorage(service.UnifiedService):
         :param metadata: Optional metadata to set on volume creation
         :param imageRef: reference to an image stored in glance
         :param source_volid: ID of source volume to clone from
-        :param source_replica: ID of source volume to clone replica
+        :param source_replica: ID of source volume to clone replica(IGNORED)
         :param scheduler_hints: (optional extension) arbitrary key-value pairs
                             specified by the client to help boot an instance
         :param multiattach: Allow the volume to be attached to more than
                             one instance
+        :param backup_id: ID of the backup
 
         :returns: Return a new volume.
         """
+        if source_replica:
+            LOG.warning("The argument `source_replica` would be ignored"
+                        " because it was removed from cinder api.")
         return self._impl.create_volume(
             size, consistencygroup_id=consistencygroup_id, group_id=group_id,
             snapshot_id=snapshot_id, source_volid=source_volid,
@@ -73,7 +79,7 @@ class BlockStorage(service.UnifiedService):
             user_id=user_id, project_id=project_id,
             availability_zone=availability_zone, metadata=metadata,
             imageRef=imageRef, scheduler_hints=scheduler_hints,
-            source_replica=source_replica, multiattach=multiattach)
+            multiattach=multiattach, backup_id=backup_id)
 
     @service.should_be_overridden
     def list_volumes(self, detailed=True):
